/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include "ObjectLocationData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string ObjectLocationData::getBaseTag() {
	return "OBJECTLOCATION";
}

std::string ObjectLocationData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectLocationData::getTemplate() {
	return "<" + getBaseTag() + "><CLASS/></" + getBaseTag() + ">";
}

std::string ObjectLocationData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING><SENSORTYPE/>"
			+ "<POLYGON/></" + getBaseTag() + ">";
}

ObjectLocationData::ObjectLocationData() :
	ObjectData() {
}

ObjectLocationData::Hypothesis::Hypothesis() :
	ObjectData::Hypothesis() {
}

ObjectLocationData::ObjectLocationData(std::string generator) :
	ObjectData(generator) {
}

ObjectLocationData::Hypothesis::Hypothesis(std::string generator) :
	ObjectData::Hypothesis(generator) {
}

void ObjectLocationData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectLocationData::Hypothesis::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT");
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectLocationData::fillInto(xmltio::Location &loc) const {

	ObjectData::fillInto(loc);
	ObjectData::sanitizeLocation(loc);

	for (vector<boost::shared_ptr<ObjectData::Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {

		try {
			Hypothesis* h0 = dynamic_cast<Hypothesis*> (it->get());

			Location hyp(Hypothesis::getTemplate(), XPath(
					Hypothesis::getBaseTag()));

			h0->fillInto(hyp);
			loc.add(hyp);
		} catch (bad_cast e) {
			continue;
		}
	}
}

void ObjectLocationData::Hypothesis::fillInto(xmltio::Location &loc) const {

	ObjectData::Hypothesis::fillInto(loc);
	loc[XPath("SENSORTYPE")] = getSensorType();

	Location polyLoc = Location(loc, "POLYGON");

	for (vector<pair<int, int> >::const_iterator it = polygon.begin(); it
			!= polygon.end(); it++) {
		Location newPoint("<POINT x=\"\" y=\"\" />", "POINT");
		newPoint[XPath("@x")] = it->first;
		newPoint[XPath("@y")] = it->second;
		polyLoc.add(newPoint);
	}
}

void ObjectLocationData::fromLocation(const xmltio::Location &loc_,
		ObjectLocationData &object) {

	ObjectData::fromLocation(loc_, object);
	object.clearHypotheses();

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_); iter; ++iter) {
		Hypothesis hyp;
		Hypothesis::fromLocation(*iter, hyp);
		object.addHypothesis(hyp);
	}
}

void ObjectLocationData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	ObjectData::Hypothesis::fromLocation(loc_, object);

	object.setSensorType(extract<string> (loc_[XPath("SENSORTYPE")]));

	vector<pair<int, int> > poly;
	for (XPathIterator iter = XPath("POLYGON/POINT").evaluate(loc_); iter; ++iter) {

		int x = extract<double> ((*iter)[XPath("@x")]);
		int y = extract<double> ((*iter)[XPath("@y")]);
		poly.push_back(pair<int, int> (x, y));
	}

	object.setPolygon(poly);
}

string ObjectLocationData::Hypothesis::getSensorType() const {
	return sensorType;
}
void ObjectLocationData::Hypothesis::setSensorType(const string &sensorType) {
	ObjectLocationData::Hypothesis::sensorType = sensorType;
}

vector<pair<int, int> > ObjectLocationData::Hypothesis::getPolygon() const {
	return polygon;
}
void ObjectLocationData::Hypothesis::setPolygon(
		const vector<pair<int, int> > &polygon) {
	ObjectLocationData::Hypothesis::polygon = polygon;
}

bool ObjectLocationData::operator==(const ObjectLocationData &other) const {

	bool isHyp = true;

	try {
		vector<Hypothesis> h0 = getLocationHypotheses();
		vector<Hypothesis> h1 = other.getLocationHypotheses();

		if (h0.size() != h1.size()) {
			return false;
		}

		vector<Hypothesis>::iterator it0 = h0.begin();
		for (vector<Hypothesis>::iterator it1 = h1.begin(); it1 != h1.end(); it1++) {

			Hypothesis hyp0 = *it0;
			Hypothesis hyp1 = *it1;

			if (!(hyp0 == hyp1)) {
				isHyp = false;
			}
			it0++;
		}

		return ObjectData::operator==(other) && isHyp;
	} catch (bad_cast e) {
		return false;
	}

}

bool ObjectLocationData::Hypothesis::operator==(const Hypothesis &other) const {

	bool isPoly = true;
	if (other.getPolygon().size() != getPolygon().size()) {
		isPoly = false;
	} else {
		for (int i = 0; i < (int) getPolygon().size(); i++) {
			if (getPolygon()[i].first != other.getPolygon()[i].first
					|| getPolygon()[i].second != other.getPolygon()[i].second) {
				isPoly = false;
			}
		}
	}
	return ObjectData::Hypothesis::operator==(other) && other.getSensorType()
			== getSensorType() && isPoly;

}

void ObjectLocationData::addHypothesis(ObjectLocationData::Hypothesis &h) {
	hypotheses.push_back(boost::shared_ptr<Hypothesis>(new Hypothesis(h)));
}

vector<ObjectLocationData::Hypothesis> ObjectLocationData::getLocationHypotheses() const {

	vector<Hypothesis> h1;

	for (vector<boost::shared_ptr<ObjectData::Hypothesis> >::const_iterator it =
			hypotheses.begin(); it != hypotheses.end(); it++) {
		try {
			Hypothesis* hyp = dynamic_cast<Hypothesis*> (it->get());
			h1.push_back(*hyp);
		} catch (bad_cast e) {
			cerr << "Cannot cast Hypothesis" << endl;
			continue;
		}
	}
	return h1;
}
