/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */

#include <cmath>

#include "PersonData.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

std::string PersonData::getBaseTag() {
	return "PERSON";
}

std::string PersonData::getTemplate() {
	return std::string(
			"<" + getBaseTag() + ">"
					+ "<STATES walking=\"\" facing=\"\" gazing=\"\" talking=\"\" dist=\"\" ang=\"\" poi=\"\" />"
					+ "<IDENT name=\"none\" id=\"\"/>"
					+ "<NEIGHBOUR dist=\"-1\" id=\"-1\"/>" + "<GENERATOR/>")
			+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}

PersonData::PersonData(std::string generator) :
		Type(generator) {

}

PersonData::PersonData() :
		Type("") {

}

float PersonData::getAngle(AngleUnit angleUnit) const {
	return UnitConverter::convert(angle, RADIAN, angleUnit);
}

float PersonData::getDistance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(distance, MILLIMETER, lengthUnit);
}

bool PersonData::isFacing() const {
	return facing;
}

float PersonData::getGaze() const {
	return gaze;
}

int PersonData::getIdentId() const {
	return identId;
}

std::string PersonData::getIdentName() const {
	return identName;
}

bool PersonData::hasNeighbour() const {
	return neighbour;
}

float PersonData::getNeighbourDistance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(neighbourDistance, MILLIMETER, lengthUnit);
}

int PersonData::getNeighbourId() const {
	return neighbourId;
}

bool PersonData::isPoi() const {
	return poi;
}

bool PersonData::isTalking() const {
	return talking;
}

bool PersonData::isWalking() const {
	return walking;
}

boost::shared_ptr<Face> PersonData::getFace() const {
	return face;
}

bool PersonData::hasFace() const {
	return face.get();
}

boost::shared_ptr<Legs> PersonData::getLegs() const {
	return legs;
}

bool PersonData::hasLegs() const {
	return legs.get();
}

boost::shared_ptr<Shirt> PersonData::getShirt() const {
	return shirt;
}

bool PersonData::hasShirt() const {
	return shirt.get();
}

void PersonData::setAngle(float angle, AngleUnit angleUnit) {
	this->angle = UnitConverter::convert(angle, angleUnit, RADIAN);
}

void PersonData::setDistance(float distance, LengthUnit lengthUnit) {
	this->distance = UnitConverter::convert(distance, lengthUnit, MILLIMETER);
}

void PersonData::setFacing(bool facing) {
	this->facing = facing;
}

void PersonData::setGaze(float gaze) {
	this->gaze = gaze;
}

void PersonData::setIdentId(int identId) {
	this->identId = identId;
}

void PersonData::setIdentName(std::string identName) {
	this->identName = identName;
}

void PersonData::setHasNeighbour(bool hasNeighbour) {
	this->neighbour = hasNeighbour;
}

void PersonData::setNeighbourDistance(float neighbourDistance,
		LengthUnit lengthUnit) {
	this->neighbourDistance = UnitConverter::convert(neighbourDistance,
			lengthUnit, MILLIMETER);
}

void PersonData::setNeighbourId(int neighbourId) {
	this->neighbourId = neighbourId;
}

void PersonData::setPoi(bool poi) {
	this->poi = poi;
}

void PersonData::setTalking(bool talking) {
	this->talking = talking;
}

void PersonData::setWalking(bool walking) {
	this->walking = walking;
}

void PersonData::setFace(boost::shared_ptr<Face> face) {
	this->face = face;
}

void PersonData::setLegs(boost::shared_ptr<Legs> legs) {
	this->legs = legs;
}

void PersonData::setShirt(boost::shared_ptr<Shirt> shirt) {
	this->shirt = shirt;
}

void PersonData::sanitizeLocation(xmltio::Location &loc) const {

}

void PersonData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("STATES/@walking")] = (isWalking() ? "YES" : "NO");
	loc[XPath("STATES/@facing")] = (isFacing() ? "YES" : "NO");
	loc[XPath("STATES/@gazing")] = getGaze() * 180.0 / M_PI;
	loc[XPath("STATES/@talking")] = (isTalking() ? "YES" : "NO");
	loc[XPath("STATES/@dist")] = getDistance(MILLIMETER);
	loc[XPath("STATES/@ang")] = getAngle(DEGREE);
	loc[XPath("STATES/@poi")] = (isPoi() ? "YES" : "NO");
	if (getIdentName() == "") {
		loc[XPath("IDENT/@name")] = "identifying";
	} else {
		loc[XPath("IDENT/@name")] = getIdentName();
	}
	loc[XPath("IDENT/@id")] = getIdentId();
	loc[XPath("NEIGHBOUR/@dist")] = getNeighbourDistance(MILLIMETER);
	loc[XPath("NEIGHBOUR/@id")] = getNeighbourId();

	if (hasFace()) {
		Location faceLoc(Face::getTemplate(), Face::getBaseTag());
		faceLoc = *(getFace());
		loc.add(faceLoc);
	}

	if (hasLegs()) {
		Location legLoc(Legs::getTemplate(), Legs::getBaseTag());
		legLoc = *(getLegs());
		loc.add(legLoc);
	}

	if (hasShirt()) {
		Location shirtLoc(Shirt::getTemplate(), Shirt::getBaseTag());
		shirtLoc = *(getShirt());
		loc.add(shirtLoc);
	}

}

void PersonData::fromLocation(const xmltio::Location &loc_, PersonData &pd) {

	string strHelpString;

	Type::fromLocation(loc_, pd);

	strHelpString = extract<string>(loc_[XPath("STATES/@walking")]);
	pd.setWalking(strcmp("NO", strHelpString.c_str()));
	strHelpString = extract<string>(loc_[XPath("STATES/@facing")]);
	pd.setFacing(strcmp("NO", strHelpString.c_str()));
	pd.setGaze(extract<float>(loc_[XPath("STATES/@gazing")]) * M_PI / 180.0);
	strHelpString = extract<string>(loc_[XPath("STATES/@talking")]);
	pd.setTalking(strcmp("NO", strHelpString.c_str()));
	pd.setDistance(extract<float>(loc_[XPath("STATES/@dist")]), MILLIMETER);
	pd.setAngle(extract<float>(loc_[XPath("STATES/@ang")]), DEGREE);
	strHelpString = extract<string>(loc_[XPath("STATES/@poi")]);
	pd.setPoi(strcmp("NO", strHelpString.c_str()));
	pd.setIdentName(extract<string>(loc_[XPath("IDENT/@name")]));
	pd.setIdentId(extract<int>(loc_[XPath("IDENT/@id")]));
	pd.setNeighbourDistance(extract<float>(loc_[XPath("NEIGHBOUR/@dist")]),
			MILLIMETER);
	pd.setNeighbourId(extract<int>(loc_[XPath("NEIGHBOUR/@id")]));
	// TODO this code could be placed in the getter
	pd.setHasNeighbour(((pd.getNeighbourId() == -1) ? false : true));

	try {
		pd.setFace(
				boost::shared_ptr<Face>(
						new Face(
								extract<Face>(
										loc_[XPath(Face::getBaseTag())]))));
		assert(pd.hasFace());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

	try {
		pd.setLegs(
				boost::shared_ptr<Legs>(
						new Legs(
								extract<Legs>(
										loc_[XPath(Legs::getBaseTag())]))));
		assert(pd.hasLegs());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

	try {
		pd.setShirt(
				boost::shared_ptr<Shirt>(
						new Shirt(
								extract<Shirt>(
										loc_[XPath(Shirt::getBaseTag())]))));
		assert(pd.hasShirt());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

}

bool airobots::PersonData::operator ==(const PersonData & other) const {

	bool a = Type::operator ==(other);

	bool b = walking == other.walking;
	bool c = facing == other.facing;
	bool d = gaze == other.gaze;
	bool e = talking == other.talking;
	bool f = distance == other.distance;
	bool g = angle == other.angle;
	bool h = poi == other.poi;
	bool i = grounded == other.grounded;
	bool j = identName == other.identName;
	bool k = identId == other.identId;
	bool l = neighbour == other.neighbour;
	bool m = neighbourDistance == other.neighbourDistance;
	bool n = neighbourId == other.neighbourId;
	bool o = *face == *(other.face);
	bool p = *legs == *(other.legs);
	bool q = *shirt == *(other.shirt);

	return a && b && c && d && e && f && g && h && i && j && k && l && m && n
			&& o && p && q;
}

