/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.annotation;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.util.LinkedList;

import org.apache.log4j.Logger;

import sun.io.Converters;

import de.unibi.airobots.btl.data.navigation.NavigationGoalData;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.*;

/**
 * 
 * @author jhelbach
 */
public class MapRegion {

	public final static String POLYGONE_TYPE = "polygon";

	public final static String Rectangle_TYPE = "rectangle";

	private String type = "";

	private String label = "";

	private String group = "";
	
	private double confidence = 1.0;

	private double height = 0;

	private LinkedList<MapPoint> points = new LinkedList<MapPoint>();

	private NavigationGoalData viewPoint;

	private Logger logger = Logger.getLogger(this.getClass());
	
	private LengthUnit iLU = LengthUnit.METER;

	public MapRegion(String type) {
		this.type = type;
	}

	public MapRegion(String type, String label) {
		this.type = type;
		this.label = label;
	}

	public MapRegion(String type, String label, String group) {
		this.type = type;
		this.label = label;
		this.group = group;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoint = viewPoint;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint,
			double height,LengthUnit lU) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoint = viewPoint;
		this.height = UnitConverter.convert(height, lU, iLU);
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint,
			double height, double confidence) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.confidence = confidence;
		this.points = points;
		this.viewPoint = viewPoint;
		this.height = height;
	}
	
	public void addPoint(int x, int y, LengthUnit lU) {
		MapPoint p = new MapPoint((double)x, (double)y, lU);
		points.add(p);
	}

	public void addPoint(double x, double y, LengthUnit lU) {
		MapPoint p = new MapPoint(x, y, lU);
		points.add(p);
	}

	public void addPoint(MapPoint point) {
		points.add(point);
	}

	public Shape getShape() {
		GeneralPath generalPath = new GeneralPath();

		boolean firstPoint = true;
		logger.debug("points:");
		for (MapPoint p : points) {
			if (firstPoint) {
				generalPath.moveTo(p.getX(iLU), p.getY(iLU));
				firstPoint = false;
			} else {
				generalPath.lineTo(p.getX(iLU), p.getY(iLU));
				logger.debug("x " + (int) p.getX(iLU) + " y " + (int) p.getY(iLU));
			}
		}

		return generalPath;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public void setConfidence(double confidence) {
		this.confidence = confidence;
	}

	public double getConfidence() {
		return confidence;
	}

	public LinkedList<MapPoint> getPoints() {
		return points;
	}

	public void setPoints(LinkedList<MapPoint> points) {
		this.points = points;
	}

	public NavigationGoalData getViewPoint() {
		return viewPoint;
	}

	public void setViewPoint(NavigationGoalData viewPoint) {
		this.viewPoint = viewPoint;
	}

	public double getHeight(LengthUnit lU) {
		return UnitConverter.convert(height,iLU,lU);
	}

	public void setHeight(double height,LengthUnit lU) {
		this.height = UnitConverter.convert(height, iLU, lU);
	}

	public void removePoint(double x, double y) {

		LinkedList<MapPoint> copy = new LinkedList<MapPoint>(points);

		for (MapPoint p : copy) {

			if (p.getX(iLU) == x && p.getY(iLU) == y) {
				points.remove(p);
			}
		}

	}

	public int getPointCount() {
		return points.size();
	}

	/**
	 * Returns {@link PositionData} for a Region. If the ViewPoint is set, the
	 * Viewpoint will be return otherwise the center of the bounding box.
	 * 
	 * @return
	 */
	public PositionData getPositionData() {

		if (viewPoint != null) {
			return viewPoint;
		}

		Rectangle2D rect = getShape().getBounds2D();

		return new PositionData(rect.getCenterX(), rect.getCenterY(), 0,
				System.currentTimeMillis(),iLU,AngleUnit.RADIAN);
	}

	public boolean contains(MapRegion region) {
		if (region == null)
			return false;

		Rectangle2D pr = region.getShape().getBounds2D();

		return getShape().contains(pr);
	}

	public boolean contains(double x, double y) {
		return getShape().contains(x, y);

	}

}
