/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.common;

import java.text.SimpleDateFormat;
import java.util.Date;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.ParentNode;

import org.apache.log4j.Logger;

/**
 * Domain class representing a timestamp with several parsing methods.
 * 
 * @author jwienke
 * @author unknown
 */
public class Timestamp {

	private static final long serialVersionUID = 1L;

	private static Logger logger = Logger.getLogger(Timestamp.class);

	private Date created;
	private Date updated;

	/**
	 * Creates a timestamp with the current system time as updated and created.
	 */
	public Timestamp() {
		final long currentTime = System.currentTimeMillis();
		this.created = new Date(currentTime);
		this.updated = new Date(currentTime);
	}

	/**
	 * Uses the given time for update and creation time.
	 * 
	 * @param created
	 *            unix timestamp in milliseconds
	 */
	public Timestamp(long created) {
		this.created = new Date(created);
		this.updated = new Date(created);
	}

	public Timestamp(long created, long updated) {
		this.created = new Date(created);
		this.updated = new Date(updated);
	}

	/**
	 * Uses the given time value for last updated time.
	 * 
	 * @param updated
	 *            unix timestamp in milliseconds
	 */
	public void setUpdated(long updated) {
		this.updated = new Date(updated);
	}

	public Date getUpdated() {
		return updated;
	}

	/**
	 * Uses the given time value for creation time.
	 * 
	 * @param created
	 *            unix timestamp in milliseconds
	 */
	public void setCreated(long created) {
		this.created = new Date(created);
	}

	public Date getCreated() {
		return created;
	}

	@Override
	public String toString() {
		SimpleDateFormat fmt = new SimpleDateFormat("HH:mm:ss.SSS d.M.y");
		return "created = " + fmt.format(created) + " [" + created.getTime()
				+ "], updated = " + fmt.format(updated) + " ["
				+ updated.getTime() + "]";
	}

	@Override
	public boolean equals(Object other) {

		if (!(other instanceof Timestamp)) {
			return false;
		}
		Timestamp otherTimestamp = (Timestamp) other;

		return (created.equals(otherTimestamp.created))
				&& (updated.equals(otherTimestamp.updated));

	}

	public static String getBaseTag() {
		return "TIMESTAMP";
	}

	public static void fromElement(Element doc, Timestamp timestamp) {
		fromElement(doc, timestamp, false);
	}

	public static void fromElement(Element doc, Timestamp timestamp,
			boolean legacyParsing) {

		Long created = null;
		Long updated = null;

		Node node;
		for (int i = 0; i < doc.getChildCount(); i++) {
			node = doc.getChild(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("INSERTED")
						|| (legacyParsing && elem.getLocalName().equals(
								"CREATED"))) {
					created = ElementParser
							.getLongAttributeValue(elem, "value");
				}
				if (elem.getLocalName().equals("UPDATED")) {
					updated = ElementParser
							.getLongAttributeValue(elem, "value");
				}
			}
		}

		// enable recovery strategies if desired
		if (legacyParsing) {

			// first error case: only of both was found
			if ((created == null) && (updated != null)) {
				logger.warn("Timestamp contains only updated value. "
						+ "Guessing created from this value:\n" + doc.toXML());
				created = new Long(updated.longValue());
			} else if ((updated == null) && (created != null)) {
				logger.warn("Timestamp contains only created value. "
						+ "Guessing updated from this value:\n" + doc.toXML());
				updated = new Long(created.longValue());
			}

			// second error case: non of both was set, try direct text values as
			// timestamp
			if ((created == null) && (updated == null)) {
				try {
					Long value = Long.valueOf(doc.getValue());
					created = new Long(value.longValue());
					updated = new Long(value.longValue());
				} catch (NumberFormatException e) {
					logger.warn("Unable to interpret text contents "
							+ "of the timestamp as long value:\n" + doc.toXML());
				}
			}

		}

		if ((created != null) && (updated != null)) {

			timestamp.created = new Date(created);
			timestamp.updated = new Date(updated);

		} else {

			throw new IllegalArgumentException(
					"Unable to parse timestamp from:\n" + doc.toXML());
		}

	}

	public void fillInto(ParentNode parent) {

		Element created = new Element("INSERTED");
		created.addAttribute(new Attribute("value", Long.toString(this.created
				.getTime())));
		parent.appendChild(created);

		Element updated = new Element("UPDATED");
		updated.addAttribute(new Attribute("value", Long.toString(this.updated
				.getTime())));
		parent.appendChild(updated);

	}

}
