/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.vision3d;

import java.util.Vector;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import nu.xom.tests.XOMTestCase;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.geometry.Point3D;

/**
 * This type carries calibrated swissranger data.
 * 
 * @author lziegler
 */
public class SRCalibratedData extends Type {

	/**
	 * The coordinates scope
	 */
	public enum Scope {
		GLOBAL, LOCAL
	};

	private Scope scope = Scope.GLOBAL;
	private static final String BASETAG = "SRCALIBRATEDDATA";
	private Vector<Point3D> data = new Vector<Point3D>();

	public SRCalibratedData() {
		super.setLegacyParsing(true);
	}

	public SRCalibratedData(Vector<Point3D> data) {
		super.setLegacyParsing(true);
		this.data = data;
	}

	public SRCalibratedData(Vector<Point3D> data, Scope scope) {
		super.setLegacyParsing(true);
		this.data = data;
		this.scope = scope;
	}

	public SRCalibratedData(Scope scope) {
		super.setLegacyParsing(true);
		this.scope = scope;
	}

	public Scope getScope() {
		return scope;
	}

	public int size() {
		return data.size();
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	/**
	 * Getter for the actual data. The points are positioned by rows. Access to
	 * a specific row and col schould be done like this:
	 * 
	 * index = row * width + col
	 * 
	 * @return A {@link Vector} containing {@link Point3D} objects for each
	 *         pixel of the SwissRanger camera.
	 */
	public Vector<Point3D> getData() {
		return data;
	}

	public static String getBaseTag() {
		return BASETAG;
	}

	public void addPoint(Point3D p) {
		data.add(p);
	}

	/**
	 * Fills a {@link SRCalibratedData} object from a given XOM {@link Element}
	 * object.
	 * 
	 * @param element
	 *            The XOM {@link Element} object containing the required
	 *            information.
	 * @param data
	 *            The object to be filled.
	 * @throws ParsingException
	 */
	public static void fromElement(Element element, SRCalibratedData data)
			throws ParsingException {

		try {
			Nodes nodes = element.query(Point3D.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {
					Point3D point = new Point3D();
					Point3D.fromElement((Element) node, point);
					data.addPoint(point);
				}
			}
			data.setScope(Scope.valueOf(element.getAttribute("scope")
					.getValue()));

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	/**
	 * Fills the contents of this {@link SRCalibratedData} object into a given
	 * {@link XOMTestCase} {@link Element} object.
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		parent.addAttribute(new Attribute("scope", scope.name()));
		for (Point3D p : data) {
			Element e = new Element(Point3D.getBaseTag());
			p.fillInto(e);
			parent.appendChild(e);
		}
	}

	@Override
	public String toString() {

		String s = "[" + getBaseTag() + "]\n";
		for (Point3D p : data) {
			s += "\t" + p.toString() + "\n";
		}
		s += "[/" + getBaseTag() + "]";
		return s;
	}
}
