/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * PointCloudTest.cpp
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>
#include <math.h>

#include "btl/data/vision3d/PointCloud.h"
#include "btl/units/LengthUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PointCloudTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {
		PointCloud original;

		Point3D p0(4.2, 1.4, 2.4, MILLIMETER);
		Point3D p1(2.7, 4.3, 7.8, MILLIMETER);
		Point3D p2(6.4, 3.9, 5.0, MILLIMETER);

		original.addPoint(p0);
		original.addPoint(p1);
		original.addPoint(p2);

		Point3D ori(0, -1, 0, MILLIMETER);
		original.setSensorOrigin(ori);

		Rotation3D rot(1, 2, 3, 4, RADIAN);
		original.setRotation(rot);

		Location originalLoc(PointCloud::getTemplate(), PointCloud::getBaseTag());

		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		PointCloud parsed = extract<PointCloud>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_CLOSE(original.getSensorOrigin().getX(MILLIMETER), parsed.getSensorOrigin().getX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getSensorOrigin().getY(MILLIMETER), parsed.getSensorOrigin().getY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getSensorOrigin().getZ(MILLIMETER), parsed.getSensorOrigin().getZ(MILLIMETER), TOLERANCE);
//		BOOST_REQUIRE_EQUAL(original.getSensorRotation(), parsed.getSensorRotation());

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("PointCloud.xml"), PointCloud::getBaseTag());
	PointCloud parsed = extract<PointCloud>(fileContents);

	BOOST_REQUIRE_EQUAL("PointCloudTester", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(127194, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(127199, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(41.3, parsed.getSensorOrigin().getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(12.1, parsed.getSensorOrigin().getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(21.84, parsed.getSensorOrigin().getZ(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(12, parsed.getPoints()[0].getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(12, parsed.getPoints()[0].getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(81, parsed.getPoints()[0].getZ(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.6, parsed.getPoints()[1].getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(51, parsed.getPoints()[1].getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(511, parsed.getPoints()[1].getZ(MILLIMETER), TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
